'----- Compiler Instructions
$CPU 80386			'compile for 80386 or better CPU
$FLOAT NPX			'use the Floating Point Unit
$OPTIMIZE SIZE            	'make a small code
$COMPILE EXE              	'compile to an EXE
$DEBUG MAP OFF            	'turn on map file generation
$DEBUG PBDEBUG OFF        	'don't include pbdebug support
$LIB COM OFF       		'turn off communications library.
$LIB CGA OFF       		'turn off CGA graphics library.
$LIB EGA OFF       		'turn off EGA graphics library.
$LIB VGA OFF        		'turn off VGA graphics library.
$LIB LPT OFF       		'turn off printer support library.
$LIB IPRINT OFF       		'turn off interpreted print library.
$ERROR BOUNDS OFF       	'turn off bounds checking
$ERROR NUMERIC OFF       	'turn off numeric checking
$ERROR OVERFLOW OFF       	'turn off overflow checking
$ERROR STACK OFF       		'turn off stack checking
$COM 0                 		'set communications buffer to nothing
$STRING 1                	'set largest string size at 1 KB
$STACK 8192              	'let's use a 8 KB stack (for sure)
$SOUND 1                 	'smallest music buffer possible
$DIM ARRAY                	'force arrays to be pre-dimensioned before
$DYNAMIC                  	'all arrays will be dynamic by default
$OPTION CNTLBREAK OFF     	'don't allow Ctrl-Break to exit program

'----- UDT for the VESA BIOS Extension Information Block.
TYPE VBEInfoBlock		'declare the VBE Information Block
  VbeSignature      AS STRING*4	'VBE Signature
  VbeVersion        AS WORD	'VBE Version
  OemStringPtr      AS DWORD    'Pointer to OEM String
  Capabilities      AS DWORD    'Capabilities of graphics environment
  VideoModePtr      AS DWORD    'Pointer to Video Mode List
  TotalMemory       AS WORD	'Number of 64 KB memory blocks

  				'the next entries are added for VBE 2.0
  OemSoftwareRev    AS WORD	'VBE implementation Software revision
  OemVendorNamePtr  AS DWORD    'Pointer to Vendor Name String
  OemProductNamePtr AS DWORD    'Pointer to Product Name String
  OemProductRevPtr  AS DWORD    'Pointer to Product Revision String
  Reserved          AS STRING*222
  				'Reserved for VBE implementation
  OemData           AS STRING*256
  				'Data Area for OEM Strings
END TYPE
DIM VBE AS VBEInfoBlock		'define VBE Information Block
VBE.VbeSignature = "VBE2"	'should be the default when the function
				'is called to indicate VBE 2.0
VBEseg?? = VARSEG(VBE.VbeSignature)
VBEoff?? = VARPTR(VBE.VbeSignature)
				'32-bit address VBE information block
videomem?? = 0			'available video memory in MB

'----- UDT for the Mode Information Block.

TYPE ModeInfoBlock		'declare the Mode Information Block
                                'mandatory information for all VBE revisions
  ModeAttributes      AS WORD   'mode attributes
  WinAAttributes      AS BYTE   'window A attributes
  WinBAttributes      AS BYTE   'window B attributes
  WinGranularity      AS WORD   'window granularity
  WinSize             AS WORD   'window size
  WinASegment         AS WORD   'window A start segment
  WinBSegment         AS WORD   'window B start segment
  WinFuncPtr          AS DWORD  'pointer to window function
  BytesPerScanLine    AS WORD   'bytes per scan line

  				'mandatory information for
                                'VBE 1.2 and above
  XResolution         AS WORD	'horizontal resolution in pixels or chars
  YResolution         AS WORD   'vertical resolution in pixels or chars
  XCharSize           AS BYTE   'character cell width in pixels
  YCharSize           AS BYTE   'character cell height in pixels
  NumberOfPlanes      AS BYTE   'number of memory planes
  BitsPerPixel        AS BYTE   'bits per pixel
  NumberOfBanks       AS BYTE   'number of banks
  MemoryModel         AS BYTE   'memory model type
  BankSize            AS BYTE   'bank size in KB
  NumberOfImagePages  AS BYTE   'number of image pages
  Reserved1           AS BYTE   'reserved for page function

  				'Direct Color fields (required for
                                'direct and YUV memory models)
  RedMaskSize         AS BYTE   'size color red mask in bits
  RedFieldPosition    AS BYTE   'lsb bit position of red mask
  GreenMaskSize       AS BYTE   'size color green mask in bits
  GreenFieldPosition  AS BYTE   'lsb bit position of green mask
  BlueMaskSize        AS BYTE   'size color blue mask in bits
  BlueFieldPosition   AS BYTE   'lsb bit position of blue mask
  RsvdMaskSize        AS BYTE   'size color reserved mask in bits
  RsvdFieldPosition   AS BYTE   'lsb bit position of reserved mask
  DirectColorModeInfo AS BYTE   'direct color mode attributes

  				'mandatory information for
                                'VBE 2.0 and above
  PhysBasePtr         AS DWORD  'physical address for flat frame buffer
  OffScreenMemOffset  AS DWORD  'pointer to start of off screen memory
  OffScreenMemSize    AS WORD   'amount of off screen memory in 1KB units
  Reserved2           AS STRING*206
  				'remainder of ModeInfoBlock
END TYPE
DIM MIB AS ModeInfoBlock	'define the block
MIBseg?? = VARSEG(MIB.ModeAttributes)
MIBoff?? = VARPTR(MIB.ModeAttributes)
				'32-bit pointer to Mode Attributes

'----- Possible VESA Modes are:
'----- 100H -  640 by  400 pixels  (256 KB video RAM)
'----- 101H -  640 by  480 pixels  (512 KB  - " -   )
'----- 103H -  800 by  600 pixels  (512 KB  - " -   )
'----- 105H - 1024 by  768 pixels  (1   MB  - " -   )
'----- 107H - 1280 by 1024 pixels  (2   MB  - " -   )

'----- The following array contains the valid 256 color VESA Modes.
DIM videomodes%(4)		'VESA Mode array
videomodes%(0) = &H100		'VESA Mode 100h
videomodes%(1) = &H101          'VESA Mode 101h
videomodes%(2) = &H103          'VESA Mode 103h
videomodes%(3) = &H105          'VESA Mode 105h
videomodes%(4) = &H107          'VESA Mode 107h

'----- It follows an user defined data structure, which contains
'----- important flags, to indicate:
'             * is the VESA Mode available
'             * is the image fit to display in the VESA Mode
TYPE CheckFlags			'declare the flags
  mode      AS INTEGER          'VESA Mode flag
  available AS INTEGER          'available flag
END TYPE
DIM check(4) AS CheckFlags	'define a UDT array

mode% = 0			'the VESA Mode
help% = 0			'help variable for mode selection
maximum% = 0			'maximal possible VESA Mode
minimum% = 0			'minimal possible VESA Mode
amount??? = 1280*1024+10	'Amount of Linear Frame Buffer(LFB) which
				'must be mapped into the application's
                                'address space. We select the measures
                                'of the best mode + a 10 byte security
                                'reserve, to avoid a GPF.
number??? = 0			'number of dwords for the selected
				'VESA Mode

'----- Strings for the VESA Mode description.
DIM vesa$(4)			'VESA Mode description array
vesa$(0) = " (640  by  400 Pixels) ===> "
vesa$(1) = " (640  by  480 Pixels) ===> "
vesa$(2) = " (800  by  600 Pixels) ===> "
vesa$(3) = " (1024 by  768 Pixels) ===> "
vesa$(4) = " (1280 by 1024 Pixels) ===> "

pattern1??? = &H04030201	'color pattern for screen fill
pattern2??? = &H0		'pattern for graphics CLS
mappedlfb??? = 0		'address of the mapped LFB

'----- Variables for the Protected Mode management.
farsize??? = 0                  'variable for far heap manipulation
dpmimajor? = 0			'DPMI major version
dpmiminor? = 0                  'DPMI minor version
flag32? = 0                     'flag for 32-bit DPMI Host
errorcode? = 0			'error code
				'Factory Default = 0 (no error)
				'do not change the value, it's SHARED

'----- Mandelbrot Set variables
startreal = -2.5		'start x-direction
endreal = 1.5			'end x-direction
startimag = -1.5		'start y-direction
endimag = 1.5			'end y-direction
stepi = 0			'step in imaginary direction
stepr = 0			'step in real direction

'----- UDT for passed procedure addresses.
TYPE Addresses                  'UDT for passed procedure addresses
  pmSetReal     AS DWORD        'PM SetReal (PM = Protected Mode address)
  rmBackLab     AS DWORD        'RM BackLab (RM = Real Mode address)
  pmWaiting     AS DWORD        'PM Waiting
  pmTextMode	AS DWORD	'PM TextMode
  pmMapReg      AS DWORD	'PM MapReg
  pmVesaMode    AS DWORD	'PM VesaMode
  pmFillScreen  AS DWORD	'PM FillScreen
  pmMandel      AS DWORD	'PM Mandel
  pmDrawPixel   AS DWORD	'PM DrawPixel
END TYPE
DIM pa AS Addresses		'passed addresses
paseg?? = VARSEG(pa.pmSetReal)	'32-bit pointer to passed addresses
paoff?? = VARPTR(pa.pmSetReal)
pa.pmSetReal = CODEPTR32(SetReal) 'Real Mode address SetReal
pa.rmBackLab = CODEPTR32(BackLab) 'Real Mode address BackLab
pa.pmWaiting = CODEPTR32(Waiting) 'Real Mode address Waiting
pa.pmTextMode = CODEPTR32(TextMode) 'Real Mode address TextMode
pa.pmMapReg = CODEPTR32(MapReg)	  'Real Mode address MapReg
pa.pmVesaMode = CODEPTR32(VesaMode) 'Real Mode address VesaMode
pa.pmFillScreen = CODEPTR32(FillScreen) 'Real Mode address FillScreen
pa.pmMandel = CODEPTR32(Mandel)	'Real Mode address Mandel
pa.pmDrawPixel = CODEPTR32(DrawPixel) 'Real Mode address DrawPixel

'----- Strings
cont$ = " Please press any key to continue..."
fin$ = " Please press any key to end the application..."
a$ = ""

'----- Main Program
$INCLUDE"LFB.INC"		'integrate INC file
CLS				'clear text screen

'----- Check the installed processor. We need at least an 80386.
IF pbvCpu? < 3 THEN		'not an 80386 or better processor
  PRINT
  PRINT" Sorry!"
  PRINT" This application needs an 80386 or better CPU."
  PRINT" Program ends now."
  END				'program ends here
END IF

'----- Is a VESA BIOS or a VESA Driver available?
IF ISFALSE GetVBE%(VBEseg??, VBEoff??) THEN 'VBE Information Block not available
  PRINT
  PRINT" Sorry!"
  PRINT" FUNCTION: Return VBE Controller Information failed."
  PRINT" Program ends now."
  END				'program ends here
END IF

videomem?? = (VBE.TotalMemory*64) 'calculate available video memory in KB

'----- We'll use the Linear Video Frame Buffer above 1 MB in
'----- the Protected Mode. That's only supported under VBE 2.0.
IF VBE.VbeVersion < &H200 THEN
  PRINT
  PRINT" Sorry!"
  PRINT" This application needs VBE 2.0."
  PRINT" Program ends now."
  END
END IF

'----- We've to test, which 256 color modes are available.
FOR i% = 0 TO 4			'VESA Mode Loop
  mode% = videomodes%(i%)	'specify VESA Mode
  IF GetMIB%(mode%,MIBseg??,MIBoff??) THEN
    check(i%).mode = 1		'set available flag
  ELSE
    check(i%).mode = 0          'mode not available
  END IF
NEXT i%

'----- Set now the flag for available/unavailable VESA Mode.
FOR i% = 0 TO 4
  IF check(i%).mode = 1 THEN
    check(i%).available = 1
  ELSE
    check(i%).available = 0
  END IF
NEXT i%

'----- Check the minimal possible VESA Mode.
FOR i% = 0 TO 4
  IF check(i%).available = 1 THEN 'mode is possible
    minimum% = i%
    EXIT FOR
  END IF
NEXT i%

'----- Check the maximal possible VESA Mode.
FOR i% = 4 TO 0 STEP -1
  IF check(i%).available = 1 THEN 'mode is possible
    maximum% = i%
    EXIT FOR
  END IF
NEXT i%

PRINT
IF (check(0).available OR check(1).available OR check(2).available _
  OR check(3).available OR check(3).available)= 1 THEN
  PRINT" Available 256 Color VESA Modes:"
  PRINT" ==============================="
  PRINT
  FOR i% = 0 TO 4
    IF check(i%).available = 1 THEN
      PRINT" Mode Number ";HEX$(videomodes%(i%));" h";vesa$(i%);" [";i%;"]"
    END IF
  NEXT i%
  i% = 100			'set variable to an impossible value
  WHILE ((i% < minimum%) OR (i% > maximum%))
    PRINT
    INPUT " Please enter the appropriate number: ",a$
    i% = ASC(a$)-&H30		'convert input into numerical value
  WEND
ELSE
  PRINT" No 256 Color VESA Modes possible."
  PRINT
  PRINT" Program ends now."
  END				'close application
END IF
CLS
mode% = videomodes%(i%)

'get Mode Information Block for the VESA Mode, which the user want to see
help% = GetMIB%(mode%,MIBseg??,MIBoff??)

'----- Can we use the Linear Frame Buffer above 1 MB?
IF ISFALSE CheckLfb%(MIB.ModeAttributes) THEN
  PRINT
  PRINT" Sorry!"
  PRINT" That VESA Mode doesn't support the Linear Frame Buffer above"
  PRINT" 1 MB. Please, try another one (check listing remarks)."
  PRINT" Program ends now."
  END
END IF

'----- Calculate the number of dwords for the appropriate VESA Mode.
number??? = MIB.XResolution*MIB.YResolution\4

'----- Calculate the steps for the Mandelbrot Set.
stepi = (endimag-startimag)/MIB.YResolution
stepr = (endreal-startreal)/MIB.XResolution

'----- Is DPMI support available?
IF ISFALSE CheckDpmi%(dpmimajor?, dpmiminor?, flag32?) THEN 'DPMI support isn't available
  PRINT
  PRINT" Sorry!"
  PRINT" Your configuration doesn't support DPMI."
  PRINT
  PRINT" This application works only in the following environments:"
  PRINT
  PRINT"  OpenDOS 7.01 or DRDOS 7.02 (DPMI=ON in the CONFIG.SYS)"
  PRINT"  OS/2 Warp (3 and 4) DOS Emulation or VMB"
  PRINT"  Linux DOS Emulation (DosEmu) Version 0.66 and above"
  PRINT"  DOS Emulation of Windows (3.x, 95, 98, NT)
  PRINT"  Borland's 32RTM.EXE is a 32-bit DPMI Host. You can use"
  PRINT"   it from the DOS command line."
  PRINT"  Try CW Sandmann's 32-bit DPMI Host CWSDPMI.EXE."
  PRINT"   It comes with source code, is free, available via"
  PRINT"   several FTP sites, is distributed under the GNU
  PRINT"   Public License, and can handle virtual memory."
  PRINT
  PRINT" Program terminates now."
  END
END IF

'----- We need 32-Bit support. That has a few advantages: A 16-Bit
'----- client can't set segment limits greater than 0FFFFh (64 KB).
IF flag32? = 0 THEN		'only 16-bit support
  PRINT
  PRINT" Sorry!"
  PRINT" The installed DPMI Host doesn't support 32-Bit applications."
  PRINT
  PRINT" Program ends now."
  END				'program ends here
END IF

'----- We must give the DPMI host memory space for his private data.
'----- The required amount of that memory area determines the
'----- FUNCTION CheckDpmi%.
farsize??? = SETMEM(-600000)	'give all memory back to DOS
farsize??? = SETMEM(50000)	'The size of our program. If your
                                'program needs more DOS memory, please
                                'change this value. You can determine
                                'the appropriate values with the PB IDE,
                                'Item "Compile" and "Get Info".

'----- The FUNCTION PmAlloc% allocates memory for the necessary
'----- private DPMI Host data. If this function fails, the program
'----- terminates, because it wouldn't work properly in the
'----- Protected Mode.
IF ISFALSE PmAlloc% THEN	'memory allocation failed
  PRINT
  PRINT" Sorry!"
  PRINT" Couldn't allocate DOS memory for private DPMI data."
  PRINT" Program ends now."
  END				'program abort
END IF

'----- Let's print some information for the user.
PRINT
PRINT" System Information:"
PRINT" ==================="
PRINT
PRINT" Installed Video Memory:            ";videomem??;"KB =";_
                                            videomem??\1024;"MB"
PRINT" VESA Mode:                          ";HEX$(mode%);" H"
PRINT" Screen Width:                      ";MIB.XResolution;"Pixels"
PRINT" Screen Height:                     ";MIB.YResolution;"Pixels"
PRINT" Color Depth:                       ";MIB.BitsPerPixel;"Bit"
PRINT" Linear Frame Buffer above 1 MB at: ";MIB.PhysBasePtr
PRINT" DPMI Host Version:                 ";dpmimajor?;".";dpmiminor?
PRINT" 32-bit clients supported:           yes"
PRINT" Private DPMI area allocated:        yes"
PRINT
PRINT cont$			'print continue message
CALL Waiting                    'wait
CLS				'clear text screen

PRINT
PRINT" IMPORTANT: Please, read that screen very carefully."
PRINT" ==================================================="
PRINT
PRINT" At this point, it is clear that your configuration supports 32-bit"
PRINT" DPMI and your graphics adaptor is VESA compliant and will handle some,"
PRINT" if not all possible 256 color modes. But, if you read that screen in"
PRINT" a DOS window - something is wrong. This program will change the"
PRINT" screen resolution for a while. And the best graphics card and monitor"
PRINT" in the world can't handle 2 resolutions at the same time: one inside,"
PRINT" the other outside the window. There's only 1 cathode ray available!"
PRINT" Therefore, change immediately to a DOS Full Screen:"
PRINT"  press ALT+ENTER under Windows"
PRINT"  press ALT+POS1 under OS/2"
PRINT"  start the DosEmu from the Linux console; don't use XDosEmu"
PRINT" After switching to the 32-bit Protected Mode, the program uses"
PRINT" the Linear Frame Buffer above 1 MB, which isn't possible in"
PRINT" V86 Mode. It will:"
PRINT" 1. Draw a color pattern on your screen"
PRINT" 2. Draw the Mandelbrot Set. That's the job of a PB procedure"
PRINT"    with a little bit Assembly Language magic."
PRINT
PRINT cont$			'print continue message
CALL Waiting                    'wait
CLS				'clear text screen

'----- At this point, we're ready to initialize the Protected Mode.
LOCATE 1,1,0			'hide cursor

'----- Initialize Protected Mode
IF ISFALSE InitPM%(paseg??, paoff??) THEN
   PRINT
   CALL ErrorHandling(errorcode?) 'print error message and terminate
END IF

'----- Map the address of the Linear Frame Buffer into the program's address space.
CALL dword pa.pmMapReg BDECL(MIB.PhysBasePtr, amount???, mappedlfb???)

'----- Set the VESA Mode.
CALL dword pa.pmVesaMode BDECL(mode%)

'----- Fill the Linear Frame Buffer with a color pattern.
CALL dword pa.pmFillScreen BDECL(number???, pattern1???, mappedlfb???)
CALL dword pa.pmWaiting		'wait

'----- Clear the graphics screen.
CALL dword pa.pmFillScreen BDECL(number???, pattern2???, mappedlfb???)

'----- Draw the Mandelbrot Set.
CALL dword pa.pmMandel BDECL(MIB.XResolution, MIB.YResolution, mappedlfb???, stepi, stepr, pa.pmDrawPixel)
CALL dword pa.pmWaiting		'wait

'----- Clear the graphics screen.
CALL dword pa.pmFillScreen BDECL(number???, pattern2???, mappedlfb???)

'----- Set the text mode.
CALL dword pa.pmTextMode

'----- Switch back to Real Mode.
'----- That avoids a GPF in the PowerBASIC RTL cleanup code.
CALL dword pa.pmSetReal

LOCATE 1,1,0			'hide cursor
BackLab:			'back jump label

PRINT
PRINT" **********************************************"
PRINT" * Back again in Real Mode without any error. *"
PRINT" **********************************************"
PRINT
PRINT" Original Linear Frame Buffer address: ";MIB.PhysBasePtr
PRINT" Mapped onto memory address:           ";mappedlfb???
PRINT" Linear Frame Buffer length:           ";amount???;"Bytes"
PRINT
PRINT" Please, send error reports or improvement suggestions to:
PRINT
PRINT" Internet: 100775.2275@compuserve.com"
PRINT
PRINT fin$			'end message
CALL Waiting			'wait
CLS                             'clear text screen
END				'end main program

'SUB ErrorHandling
'Task:		Check the error code, print the appropriate error
'               message and terminate.
'Input:         pcode? = error code
'Output:        error message
SUB ErrorHandling(BYVAL pcode?)
  PRINT
  PRINT
  SELECT CASE pcode?	        'check error code
    CASE = 1			'Protected Mode initialization failed
      PRINT" Couldn't enter 32-bit Protected Mode."
    CASE = 2			'descriptor allocation failed
      PRINT" Couldn't allocate needed descriptors."
    CASE = 3			'descriptor write error
      PRINT" Couldn't write 32-bit data descriptor."
    CASE = 4
      PRINT" Couldn't write return descriptor."
    CASE = 5
      PRINT" Couldn't map linear region."
    CASE = 6
      PRINT" Couldn't set the selected VESA Mode."
      PRINT"  Please try another one (check listing remarks)."
  END SELECT
  PRINT"  Application terminates now."
  END				'end application
END SUB

'SUB Mandel
'Task:		Draw the Mandelbrot Set in a VESA Mode.
'Input:		x??        = screen width
'               y??        = screen height
'               mlfb???    = mapped Linear Frame Buffer
'               istep      = step in imaginary direction
'               rstep      = step in real direction
'               address??? = Protected Mode procedure address
'                            for the pixel set routine
'Output:	Mandelbrot Set on the screen
'Notes:         1. The pixel set routine is useable in every 256
'                  color VESA Mode, using the LFB. To be generic enough,
'                  it needs not only the x- and y-coordinate and the
'                  pixel's color, but also the start of the LFB and
'                  the length of 1 scan line in the appropriate mode.
'               2. There is made no attempt, to improve the calculations,
'                  for example with integer arithmetic etc. It's simply
'                  PB code, which is called from the Protected Mode.
'                  Only the bailout condition inside the iteration loop
'                  is normally:
'                  IF qimag+qreal > 4 THEN
'                     h% = k% MOD 256
'                     EXIT FOR
'                  END IF
'                  That was re-written with more effective FPU code.
SUB Mandel(x??,y??,mlfb???,istep,rstep,address???)
  bailcond = 4.0		'bailout condition
  startimag = -1.5
  startreal = -2.5
  zreal = 0			'real part z
  zimag = 0			'imaginary part z
  creal = 0			'real part c
  cimag = 0			'imaginary part c
  qreal = 0			'real part q
  qimag	= 0			'imaginary part q
  deep% = 100			'number of iterations
  h% = 0			'color
  i% = 0			'column counter
  j% = 0			'row counter
  k% = 0			'iteration counter
  numcol% = x??

  FOR i% = 0 TO x??-1		'column loop
    FOR j% = 0 TO y??-1         'row loop
      cimag = istep*j%+startimag
      creal = rstep*i%+startreal
      zimag = cimag		'start iteration with z = c
      zreal = creal
      qimag = zimag*zimag	'that's important for the bailout
      qreal = zreal*zreal
      FOR k% = 1 TO deep%	'iteration loop
        h% = 0			'important for the colors
        zimag = 2*zreal*zimag+cimag
        zreal = qreal-qimag+creal
        qimag = zimag*zimag
        qreal = zreal*zreal
        ! lea	bx,bailcond	;ss:bx -> bailcond
        ! db	&H36		;fld	dword ptr ss:[bx]
        ! db	&HD9		;st(0) = bailcond
        ! db	&H07
        ! lea	bx,qreal	;ss:bx -> qreal
        ! db	&H36		;fld	dword ptr ss:[bx]
        ! db	&HD9		;st(0) = qreal
        ! db	&H07		;st(1) = bailcond
        ! lea	bx,qimag	;ss:bx -> qimag
        ! db	&H36		;fadd	dword ptr ss:[bx]
        ! db	&HD8		;st(0) = qreal+qimag
        ! db	&H07		;st(1) = bailcond
        ! db	&HDE		;fcompp
        ! db	&HD9		;compare st(0) with st(1)
        ! db	&H9B		;fstsw	ax
        ! db	&HDF		;load FPU status into ax
        ! db	&HE0
        ! db	&H9E		;sahf
        !			;make it as CPU flags
        ! ja	label1		;st(0) > st(1)
        ! jmp	label2		;st(0) <= st(1)
        label1:
        ! mov	ax,k%		;ax = iteration number
        ! and	ax,&H00ff	;that is k% MOD 256
        ! mov	h%,ax		;save it as color value
        ! mov	ax,deep%	;ax = loop end value
        ! mov	k%,ax		;save it in loop counter
        label2:			'next iteration
      NEXT k%

      '----- Set the pixel with the appropriate color.
      CALL dword address??? BDECL(numcol%, i%, j%, h%, mlfb???)

    NEXT j%
  NEXT i%
END SUB
