                .386p                   ;80386 privileged instructions
                LOCALS  @@              ;block-scoped symbols
                JUMPS                   ;jump optimization

;----- EXTERNALS

                extrn	errorcode:BYTE	;program's error code
                extrn	ErrorHandling:FAR
                			;handle error messages

;----- PUBLICS
                                        ;RM = Real Mode
                                        ;PM = Protected Mode
                public	Waiting		;wait for key (PM or RM)
                public	GetVBE		;fill VBE Information Block (RM)
                public	GetMIB		;fill Mode Information Block (RM)
                public	CheckLfb	;check Linear Frame Buffer (RM)
                public	CheckDpmi	;check DPMI support (RM)
                public	PmAlloc		;allocate DOS memory (RM)
                public	SegOff2Lin	;convert segment:offset into
                			;linear address (RM)
		public	InitPM		;initialize Protected Mode (RM)
                public	VesaMode	;set VESA Mode (PM)
                public	TextMode	;set text mode (RM or PM)
                public	MapReg		;map linear region (PM)
                public	FillScreen	;fill the Linear Frame Buffer
                			;with a color pattern
                public	DrawPixel	;set a pixel in the VESA Mode
                public	SetReal		;set Real Mode (PM)

DATA		segment	word use16 public 'DATA'

					;code descriptor
dcode           db      0ffh            ;segment length 64 KB
		db      0ffh
		db      0               ;start address
		db      0               ;(patched at run time)
		db      0
		db      9ah             ;access rights (patched)
		db      0               ;byte granular
		db      0
					;32-bit data descriptor
ddata32         db      0ffh            ;segment length 4 GB
		db      0ffh
		db      0               ;start address
		db      0
		db      0
		db      92h             ;access rights (patched)
		db      0cfh            ;page granular
		db      0

pmtorm          df      ?               ;selector:offset Protected -> Real
rmtopm		dd	?		;segment:offset Real -> Protected
dpmi            dd      ?               ;far pointer to DPMI mode switch
					;entry point
pdata           dw      ?               ;number of paragraphs for private
					;DPMI data
segpd           dw      ?               ;segment DPMI private data area
rmDS            dw      ?               ;real mode DS
rmES            dw      ?               ;real mode ES
rmSS            dw      ?               ;real mode SS
rmCS            dw      ?               ;real mode CS
rmSP            dw      ?               ;real mode SP
rmIP            dw      ?               ;real mode IP
selinc		dw	?		;selector increment value
seldata		dw	?		;4 GB data selector
selret		dw	?		;return selector
linpar		dd	?		;linear address passed
					;parameters
mylab		dd	?		;return address of (Label in
					;PB main program)

DATA		ends

CODE_SEG        segment word public use16 'CODE'
                assume  cs:CODE_SEG,ds:DATA

;SUB Waiting
;Task:          Wait, until the user has pressed any key.
;Input:         None
;Output:        waiting
;Uses:          BIOS Interrupt 16h
;Registers:     None
;Note:		BIOS Interrupts work as well in Real Mode as
;               in the Protected Mode.
Waiting         proc    far
		push    ax
		xor     ax,ax           ;FUNCTION: wait for key
					;(ah = 0)
		int     16h             ;transfer to BIOS
		pop     ax
		ret
Waiting         endp

;FUNCTION GetVBE%
;Task:		Fill the VBE information block
;Input:         [bp+6]  = offset address VBE information block
;               [bp+8]  = segment address VBE information block
;Output:	TRUE    ===> VBE information block filled
;               FALSE   ===> Function failed
;Uses:		VESA Service
GetVBE		proc	far
@@off		=	[bp+6]		;offset VBE block
		push	bp
                mov	bp,sp
                push	es
		push	di
                les	di,@@off	;es:di -> VBE information block
                mov	ax,4f00h	;function: Return VBE information
                int	10h		;transfer to VESA driver
                cmp	al,4fh		;function supported?
                jne	@@20		;no: jump
                or	ah,ah		;function succesful?
                jnz	@@20		;no: jump
                mov	ax,1		;yes: set function result TRUE
@@10:           pop	di
		pop	es
                pop	bp
                ret	4		;return to caller
@@20:		xor	ax,ax		;set function result FALSE
		jmp	short @@10	;return to caller
GetVBE		endp

;FUNCTION GetMIB%
;Task:		Get the Mode Information Block.
;Input:         [bp+6]  = offset address Mode Information Block
;               [bp+8]  = segment address Mode Information Block
;               [bp+10] = vesa mode number
;Output:	TRUE    ===> Mode Information Block filled
;               FALSE   ===> failed
;Uses:		VESA Service
GetMIB		proc	far
@@off		=	[bp+6]		;offset Mode Information Block
@@mode		=	[bp+10]		;VESA Mode number
		push	bp
                mov	bp,sp
                push	es
                push	cx
                push	di
                mov	ax,4f01h	;FUNCTION: get mode information
                mov	cx,@@mode	;cx = vesa mode number
                les	di,@@off	;es:di -> Mode Information Block
                int	10h		;transfer to vesa driver
                cmp	al,4fh		;function supported?
                jne	@@20		;no: jump
                or	ah,ah		;function succesful?
                jnz	@@20		;no: jump
                mov	ax,1		;yes: set function result TRUE
@@10:           pop	di
		pop	cx
                pop	es
                pop	bp
                ret     6
@@20:		xor	ax,ax		;set function result FALSE
		jmp	short @@10
GetMIB		endp

;FUNCTION CheckLfb%
;Task:		Check the usage of the Linear Frame Buffer above
;		1 MB.
;Input:		[bp+6]  = Mode Attributes
;Output:	TRUE    ===> Linear Frame Buffer available
;               FALSE   ===> Linear Frame Buffer not available
CheckLfb	proc	far
@@attr		=	[bp+6]		;Mode Attribute flag
		push	bp
                mov	bp,sp
                test	word ptr @@attr,80h
                			;Linear Frame Buffer available?
                jz	@@20		;no: return FALSE
                mov	ax,1		;yes: return TRUE
@@10:           pop	bp
                ret	2
@@20:		xor	ax,ax		;set FUNCTION result FALSE
		jmp	short @@10	;and terminate
CheckLfb	endp

;FUNCTION CheckDpmi%
;Task:          Check DPMI support.
;Input:         [bp+6]  = address 32-bit support flag
;               [bp+10] = address DPMI minor version
;               [bp+14] = address DPMI major version
;Output:        TRUE    ===> DPMI support available
;                       passed variables filled with appropriate values
;               FALSE   ===> DPMI support isn't available
;                       values of passed variables meaningless
;Uses:          DOS Multiplex Interrupt 2Fh
CheckDpmi       proc    far
@@flagadd	=	[bp+6]		;address 32-bit flag
@@minoradd	=	[bp+10]		;address minor version
@@majoradd	=	[bp+14]		;address major version
		push    bp
		mov     bp,sp
		push    ds
		push    es
		push    bx
		push    cx
		push    dx
		push    di
		push    si
		mov     ax,1687h        ;FUNCTION: check DPMI support
		int     2fh             ;transfer to DOS
		or      ax,ax           ;success?
		jnz     @@20            ;no: bail out
		mov     word ptr dpmi,di
		mov     word ptr dpmi+2,es
					;save DPMI entry point
		mov     pdata,si        ;save number of paragraphs for
					;private DPMI data
		lds     si,@@flagadd    ;ds:si -> 32-bit flag
		les     di,@@minoradd   ;es:di -> DPMI minor version
		and     bx,01h          ;isolate bit 0
		mov     byte ptr ds:[si],bl
					;save 32-bit flag
		mov     byte ptr es:[di],dl
					;save DPMI minor version
		lds     si,@@majoradd   ;ds:si -> DPMI major version
		mov     byte ptr ds:[si],dh
					;save DPMI minor version
		mov     ax,1            ;set Function result TRUE
@@10:           pop     si
		pop     di
		pop     dx
		pop     cx
		pop     bx
		pop     es
		pop     ds
		pop     bp
		ret     12
@@20:           xor     ax,ax           ;set Function result FALSE
		jmp     short @@10      ;return to caller with FALSE
CheckDpmi       endp

;FUNCTION PmAlloc%
;Task:          Allocate memory for private DPMI data.
;Input:         None
;Output:        TRUE  ===> memory allocated
;               FALSE ===> couldn't allocate memory
;Uses:          DOS Interrupt 21h
PmAlloc         proc    far
		push    bx
		mov     bx,pdata        ;bx = paragraphs required
		mov     ah,48h          ;function: allocate memory
		int     21h             ;transfer to DOS
		jc      @@20            ;error: jump
		mov     segpd,ax        ;save segment for private DPMI data
		mov     ax,1            ;set function result TRUE
@@10:		pop     bx
                ret
@@20:           xor     ax,ax           ;set function result FALSE
		jmp     short @@10      ;return with error
PmAlloc         endp

;SUB SegOff2Lin
;Task:		Convert segment:offset into linear address.
;Input:		[bp+4] = offset to convert
;               [bp+6] = segment to convert
;               [bp+8] = offset linear address
;Output:	linear address starting at 0
SegOff2Lin	proc	far
@@off		=	[bp+6]		;offset to convert
@@seg		=	[bp+8]		;segment to convert
@@dest		=	[bp+10]		;destination of result
		push	bp
                mov	bp,sp
                push	es
                push	eax
                push	bx
                push	ecx
                les	bx,@@dest	;es:bx -> destination address
                xor	eax,eax		;eax = 0
                xor	ecx,ecx		;ecx = 0
                mov	ax,@@seg	;eax = segment
                mov	cx,@@off	;ecx = offset
                shl	eax,4		;eax = segment*16
                add	eax,ecx		;eax = segment*16+offset
                			;= linear address
                mov	dword ptr es:[bx],eax
                			;store linear address
                pop	ecx
                pop	bx
                pop	eax
                pop	es
                pop	bp
                ret	8
SegOff2Lin	endp

;FUNCTION InitPM%
;Task:		Initialize 32-bit Protected Mode.
;Input:		[bp+6] = offset passed values UDT
;               [bp+8] = segment passed values UDT
;Output:        TRUE   ===> program in 32-bit Protected Mode
;               FALSE  ===> failed, program in Real Mode
;Uses:          Protected Mode Interrupt 31h
InitPM		proc	far
@@ip		=	[bp+2]		;offset return address
@@cs		=	[bp+4]		;segment return address
@@poff		=	[bp+6]		;offset procedure array
@@pseg		=	[bp+8]		;segment procedure array
		push	bp
                mov	bp,sp
                push	ebx
                push	ecx
                push	edx
                call	SaveReal	;save some Real Mode registers
                call	SegOff2Lin PASCAL,ds,offset linpar, \
			word ptr @@pseg,word ptr @@poff
                			;convert segment:offset of passed
                			;array into linear address
                xor	eax,eax		;eax = 0
                mov	ax,@@cs		;eax = segment return address
                shl	eax,4		;convert into linear address
                or	dword ptr dcode+2,eax
                			;patch start address in
                                        ;code descriptor
		mov     es,segpd        ;es = segment private DPMI data
		mov     ax,1            ;indicate 32-bit client
		call    dpmi            ;switch to Protected Mode
                jnc	@@10		;success: jump
                mov	byte ptr errorcode,1
                			;error 1: couldn't enter
                                        ;32-bit Protected Mode
                xor	ax,ax		;ax = FALSE
                jmp	InitEnd		;terminate: program in Real Mode
@@10:		call	GetSwitch	;determine raw switch addresses
		mov	ax,0003h	;FUNCTION: get next selector
                			;increment value
                int	31h		;transfer to DPMI Host
                			;(this Function always succeeds)
		mov	selinc,ax	;save this value
                xor	ax,ax		;FUNCTION: allocate descriptor
                mov	cx,2		;cx = number of descriptors
                int	31h		;transfer to DPMI Host
                jnc	@@20		;success: jump
                call	RealMode	;switch to Real Mode
                mov	byte ptr errorcode,2
                			;error 2: couldn't allocate
                                        ;LDT descriptors
                xor	ax,ax		;ax = FALSE
                jmp	InitEnd		;terminate: program in Real Mode
@@20:           mov	seldata,ax	;save 32-bit data selector
		add	ax,selinc	;calculate descriptor for
                			;return address
                mov	selret,ax	;save return selector
		mov     ax,cs           ;set DPL field of descriptor
		lar     ax,ax           ;! Protected Mode Instruction !
		and     ah,60h
		or      byte ptr ddata32+5,ah
                or	byte ptr dcode+5,ah
					;patch access rights
                call	WriteDesc PASCAL,offset ddata32,seldata
		                        ;write 4 GB data descriptor
                jnc	@@30		;success: jump
                call	RealMode	;switch to Real Mode
                mov	byte ptr errorcode,3
                			;error 3: couldn't write 32-bit
                                        ;code descriptor
                xor	ax,ax		;ax = FALSE
                jmp	InitEnd		;terminate: program in Real Mode
@@30:           call	WriteDesc PASCAL,offset dcode,selret
		                        ;write code descriptor for
                			;return address
                jnc	@@40		;success: jump
                call	RealMode	;switch to Real Mode
                mov	byte ptr errorcode,4
                			;error 3: couldn't write
                                        ;code descriptor
                xor	ax,ax		;ax = FALSE
                jmp	InitEnd		;terminate: program in Real Mode

;----- At this point, we have to change the segments in the
;----- procedure addresses into valid Protected Mode selectors.

@@40:           mov	ax,seldata	;ax = 32-bit data selector
                mov	gs,ax		;cache data selector in gs
                mov	ebx,linpar	;gs:ebx -> array
                mov	ax,cs		;ax = current code selector
                mov	word ptr gs:[ebx+2],ax
                			;write selector for SetReal
                mov	word ptr gs:[ebx+10],ax
                			;write selector for Waiting
                mov	word ptr gs:[ebx+14],ax
                			;write selector for TextMode
                mov	word ptr gs:[ebx+18],ax
                			;write selector for MapReg
                mov	word ptr gs:[ebx+22],ax
                			;write selector for VesaMode
		mov	word ptr gs:[ebx+26],ax
                			;write selector for FillScreen
                mov	word ptr gs:[ebx+34],ax
                			;write selector DrawPixel
                mov	ax,selret	;ax = selector PB code segment
                mov	@@cs,ax		;store PB code selector
                			;in return address
                mov	word ptr gs:[ebx+30],ax
                			;write selector for Mandel
                mov	eax,gs:[ebx+4]	;eax = Real Mode address BackLab
                mov	mylab,eax	;store it in mylab
		mov	ax,1		;ax = TRUE
InitEnd:	pop	edx
		pop	ecx
		pop	ebx
                pop	bp
                ret	4
InitPM		endp

;SUB VesaMode
;Task:		Set the selected VESA Mode.
;Input:		[bp+6]  = address VESA Mode
;Output:	Success:  VESA Mode set
;               Failed:  program again in Real Mode, error message
VesaMode	proc	far
@@mode		=	[bp+6]		;address VESA Mode
		push	bp
                mov	bp,sp
                push	ax
                push	bx
                push	si
                push	es
                les	si,@@mode	;es:si -> VESA Mode
                mov	bx,es:[si]	;bx = VESA Mode
                or	bx,4000h	;use Linear Frame buffer
                mov	ax,4f02h	;FUNCTION: set VESA Mode
                int	10h		;transfer to VESA driver
                cmp	al,4fh		;function supported?
                jne	@@10		;no: jump
                or	ah,ah		;Mode supported?
                jnz	@@10		;no: jump
                pop	es
                pop	si
                pop	bx
                pop	ax
                pop	bp
                ret	4
@@10:		call	FreeDesc PASCAL,seldata
					;free 4 GB descriptor
		call	FreeDesc PASCAL,selret
                			;free 64 KB code descriptor
		call	RealMode	;set Real Mode
                mov	errorcode,6	;write error code
                pop	es		;restore registers
                pop	si
                pop	bx
                pop	ax
                pop	bp
                add	sp,8		;adjust stack
                xor	ax,ax		;ax = 0
                mov	al,errorcode	;ax = error code
                call	ErrorHandling PASCAL,ax
                			;print error message and terminate
VesaMode	endp

;SUB TextMode
;Task:		Set the text mode (80 columns, 25 rows).
;Input:		None
;Output:	Text Mode
;Uses:		BIOS Interrupt 10h
TextMode	proc	far
		push	ax
                mov	ax,0003h	;FUNCTION: set text mode
                int	10h
                pop	ax
                ret
TextMode	endp

;SUB MapReg
;Task:		Map linear region into applications address space.
;Input:		[bp+6]  = offset map result
;		[bp+8]  = selector map result
;               [bp+10] = offset region size
;		[bp+12] = selector region size
;		[bp+14] = offset physical address to map
;		[bp+16] = selector physical address to map
;Output:	Success: address mapped into the application's
;                        address space.
;               Failed:  program again in Real Mode, error message
MapReg		proc	far
@@resoff	=	[bp+6]		;offset map result
@@regoff	=	word ptr [bp+10];offset region size
@@regsel	=	word ptr [bp+12];selector region size
@@physoff	=	word ptr [bp+14];offset physical address
@@physsel	=	word ptr [bp+16];selector physical address
		push	bp
                mov	bp,sp
                push	eax
                push	bx
                push	cx
                push	di
                push	si
                push	es
                xor	eax,eax		;eax = 0
                mov	ax,@@regsel	;ax = selector region
                mov	es,ax		;es = ax
                mov	ax,@@regoff	;es:eax -> region to map
                mov	di,es:[eax]
                mov	si,es:[eax+2]	;si:di = size of region to
                			;        map in bytes
                mov	ax,@@physsel	;ax = selector physical address
                mov	es,ax		;es = ax
                mov	ax,@@physoff	;es:eax -> physical address
                mov	cx,es:[eax]
                mov	bx,es:[eax+2]	;bx:cx = physical address to map
                mov	ax,0800h	;FUNCTION: map physical region
                int	31h		;transfer to DPMI Host
                jc	@@10		;failed: terminate
                les	di,@@resoff	;es:di -> map result
                mov	word ptr es:[di],cx
                mov	word ptr es:[di+2],bx
                			;write result
                pop	es
                pop	si
                pop	di
                pop	cx
                pop	bx
                pop	eax
                pop	bp
                ret	12
@@10:		call	FreeDesc PASCAL,seldata
					;free 4 GB descriptor
		call	FreeDesc PASCAL,selret
                			;free 64 KB code descriptor
		call	RealMode	;set Real Mode
                mov	errorcode,5	;write error code
                pop	es		;restore registers
                pop	si
                pop	di
                pop	cx
                pop	bx
                pop	eax
                pop	bp
                add	sp,16		;adjust stack
                xor	ax,ax		;ax = 0
                mov	al,errorcode	;ax = error code
                call	ErrorHandling PASCAL,ax
                			;print error message and terminate
MapReg		endp

;SUB FillScreen
;Task:		Fill the Linear Frame Buffer with a color pattern.
;		Useable too for graphics CLS. The color pattern
;               is in that case simply 0 (black).
;Input:		[bp+6]  = mapped Linear Frame Buffer address
;               [bp+10] = address color pattern
;		[bp+14] = address number of dwords
;Output:	Linear Frame Buffer filled with color pattern
FillScreen	proc	far
@@mapad		=	[bp+6]		;mapped LFB address
@@colad		=	[bp+10]		;address color pattern
@@numad		=	[bp+14]		;address dword number
		push	bp
                mov	bp,sp
                push	eax
                push	ebx
                push	ecx
                push	si
                push	es
                les	si,@@numad	;es:si -> dword number
                mov	ecx,es:[si]	;ecx = dword number (counter)
                les	si,@@mapad	;es:si -> mapped LFB
                mov	ebx,es:[si]	;gs:ebx -> Linear Frame Buffer
                les	si,@@colad	;es:si -> color pattern
                mov	eax,es:[si]	;eax = color pattern
@@10:		mov	dword ptr gs:[ebx],eax
					;write 1 dword
                add	ebx,4		;update pointer
                dec	ecx		;decrement loop counter
                jne	@@10		;close the loop
                pop	es
                pop	si
                pop	ecx
                pop	ebx
                pop	eax
                pop	bp
                ret	12
FillScreen	endp

;SUB DrawPixel
;Task:		Draw a pixel in a 8-bit VESA Mode.
;               Useable in modes: 100h, 101h, 103h, 105h, 107h.
;Input:		[bp+6]  = mapped Linear Frame Buffer addres
;		[bp+10] = address pixel color
;		[bp+14] = address current row
;		[bp+18] = address current column
;		[bp+22] = address number of columns
;Output:	Pixel on the screen
DrawPixel	proc	far
@@lfbad		=	[bp+6]		;mapped LFB address
@@colorad	=	[bp+10]		;color address
@@rowad		=	[bp+14]		;address current row
@@colad		=	[bp+18]		;address current column
@@numcolad	=	[bp+22]		;address column number
		push	bp
                mov	bp,sp
                push	eax
                push	ebx
                push	ecx
                push	si
                push	es
                xor	eax,eax		;eax = 0
                xor	ecx,ecx		;ecx = 0
                les	si,@@lfbad	;es:si -> LFB
                mov	ebx,es:[si]	;gs:ebx -> start LFB
                les	si,@@numcolad	;es:si -> number of columns
                mov	ax,es:[si]	;eax = number of columns
                les	si,@@rowad	;es:si -> row (y)
                mov	cx,es:[si]	;ecx = y
                imul	eax,ecx		;eax = y*column number
                les	si,@@colad	;es:si -> column (x)
                mov	cx,es:[si]	;ecx = x
                add	eax,ecx		;eax = y*column number+x
                			;    = offset inside LFB
                add	ebx,eax		;gs:ebx -> pixel to set
                les	si,@@colorad	;es:si -> pixel color
                mov	al,es:[si]	;al = color
                mov	byte ptr gs:[ebx],al
                			;set pixel
                pop	es
                pop	si
                pop	ecx
                pop	ebx
                pop	eax
                pop	bp
                ret	20
DrawPixel	endp

;SUB SetReal
;Task:		Switch back to Real Mode, to end the program.
;               That avoids a GPL in the RTL cleanup code.
;Input:		None
;Output:	processor again in Real Mode
;Uses:		RealMode
SetReal		proc	far
@@cs		=	[bp+4]		;return selector
		push	bp
                mov	bp,sp
                push	ax
                mov	ax,word ptr mylab+2
                			;ax = return segment
                mov	@@cs,ax		;change return address
                call	FreeDesc PASCAL,seldata
                			;free 4 GB data descriptor
                call	FreeDesc PASCAL,selret
                			;free 64 KB code descriptor
		call	RealMode	;switch back to Real Mode
                pop	ax
                pop	bp
                ret
SetReal		endp

;----- Internal Procedures

;SUB SaveReal
;Task:          Save some real mode registers.
;Input:         none
;Output:        none
SaveReal        proc    near
		mov     rmDS,ds         ;save register values
		mov     rmES,es
		mov     rmSS,ss
		mov     rmCS,cs
		ret
SaveReal        endp

;SUB GetSwitch
;Task:          Determine the addresses of the raw switch procedures.
;Input:         None
;Output:        procedure to switch from Protected Mode to Real Mode
;               procedure to switch from Real mode to Protected mode
;Uses:          Protected Mode Interrupt 31h
;Registers:     None
GetSwitch       proc    near
		push    ax
		push    bx
		push    cx
		push    edi
		push    si
		mov     ax,306h         ;FUNCTION: get addresses of
					;raw switch procedures
		int     31h             ;transfer to DPMI host
		jc      @@20            ;error: jump
		mov     word ptr rmtopm,cx
					;offset Real Mode switch
		mov     word ptr rmtopm+2,bx
					;segment Real Mode switch
		mov     dword ptr pmtorm,edi
					;offset Protected Mode switch
		mov     word ptr pmtorm+4,si
					;selector Protected Mode switch
@@20:           pop     si
		pop     edi
		pop     cx
		pop     bx
		pop     ax
		ret
GetSwitch       endp

;SUB RealMode
;Task:		Switch back to Real Mode.
;Input:		None
;Output:	processor again in Real Mode
;Uses:          DPMI Host's Raw Switch Procedure
RealMode        proc    near
		push    ax
		push    bx
		push    cx
		push    dx
		push    di
		push    si
		mov     rmSP,sp         ;save stack pointer
		mov     bx,offset @@10  ;real mode ip
		mov     rmIP,bx         ;save new ip
		mov     ax,rmDS         ;set values for real mode
		mov     cx,rmES
		mov     dx,rmSS
		mov     bx,rmSP
		mov     si,rmCS
		mov     di,rmIP
		jmp     pmtorm          ;switch back to real mode
@@10:           pop     si
		pop     di
		pop     dx
		pop     cx
		pop     bx
		pop     ax
		ret
RealMode        endp

;SUB WriteDesc
;Task:          Write a descriptor.
;Input:         [bp+6] = offset descriptor
;               [bp+4] = selector value
;Output:        cf = 0 ===> success, descriptor written
;               cf = 1 ===> failure
WriteDesc       proc    near
@@selector	=	[bp+4]		;selector value
@@descoff	=	[bp+6]		;offset descriptor
		push    bp
		mov     bp,sp
		push    ax
		push    bx
		push    edi
		push    es
		xor     edi,edi         ;edi = 0
		push    ds              ;set es = ds
		pop     es
		mov     ax,0ch          ;FUNCTION: set descriptor
		mov     bx,@@selector   ;bx = selector
		mov     di,@@descoff    ;es:edi -> descriptor
		int     31h             ;transfer to DPMI Host
		pop     es
		pop     edi
		pop     bx
		pop     ax
		pop     bp
		ret     4
WriteDesc       endp

;SUB FreeDesc
;Task:		Free an allocated descriptor.
;Input:		[bp+4]  = appropriate selector
;Output:	descriptor free
FreeDesc	proc	near
@@selector	=	[bp+4]		;selector
		push	bp
                mov	bp,sp
                push	ax
                push	bx
                mov	ax,0001h	;FUNCTION: free descriptor
                mov	bx,@@selector	;bx = selector value
                int	31h		;transfer to DPMI Host
                pop	bx
                pop	ax
                pop	bp
                ret	2
FreeDesc	endp

CODE_SEG        ends
                end
